@@ -30,15 +30,8 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#ifdef _WIN32
-#include <getopt.h>
-#endif
-
 #include <string.h>
 
-extern int optind;
-extern char *optarg;
-
 static char *prog;
 
 static void xusage()
